/* alf.h */
/* Written by, Soni. */

typedef	unsigned short int    ELF32_Half;

typedef  unsigned char        __U8;  
typedef  unsigned int         __U32; /* Elf32_Addr, Elf32_Word, Elf64_Word */
                                     /* Elf32_Off */
typedef  unsigned short int   __U16; /* Elf32_Half */
typedef  int                  __S32; /* Elf32_Sword */
typedef  unsigned long        __U64; /* Elf64_Addr
typedef  short int            __S16; /* Elf64_SHalf */
typedef  long                 __S64; /* Elf64_Sxword */

/* indices into the e_ident[] array which hold ID string of a true ELF file */
/* Alf, only verifies these indices, so Alf can easily be fooled :) */
#define EI_MAG0                0           /* File Identification */
#define EI_MAG1                1           /* File Identification */
#define EI_MAG2                2           /* File Identification */
#define EI_MAG3                3           /* File Identification */

/* indices into the e_ident[] which hold info and need parsing */
#define EI_CLASS               4           /* File Class */
#define EI_DATA		       5           /* Data Encoding */
#define EI_VERSION             6           /* File Version */
#define EI_OSABI               7           /* OS/ABI identification */
#define EI_ABIVERSION          8           /* ABI Version */
#define EI_PAD		       9           /* Start o padding bytes */ 

/* The ELFCLASS* macros are the defined values of e_ident[EI_CLASS] */
#define ELFCLASSNONE           0           /* Invalid class */
#define ELFCLASS32 	       1	   /* 32-bit objects */
#define ELFCLASS64             2           /* 64-bit objects */
#define ELFCLASS4_A	       200         /* 64-bit objects */

/* The ELFDATA* macros are the allowed values of e_ident[EI_DATA] */
#define ELFDATANONE	       0           /* Invalid data encoding */
#define ELFDATA2LSB            1           /* little Endian, 2's complement */
#define ELFDATA2MSB            2           /* big endian, 2's complement */  

/* The ELFOSABI* macros are the allowed values of e_ident[EI_OSABI] */
#define ELFOSABI_SYSV	       0
#define ELFOSABI_HPUX          1
#define ELFOSABI_STANDALONE    255

/* The ELFOSIABI* macros are allowed values of e_ident[EI_ABIVERSION] */
#define ELFABI_HPUX_NONE       0           /* None specified */ 
#define ELFABI_HPUX_REL11      1


#define EI_NIDENT              0x00000010

/* e_version , values of e_ident[EI_VERSION] */
#define EV_NONE 	0x00000000
#define EV_CURRENT	0x00000001
#define EV_NUM		0x00000002


typedef struct{
   __U8     e_ident[EI_NIDENT];
   __U16    e_type;
   __U16    e_machine;
   __U32    e_version;
   __U64    e_entry;
   __U64    e_phoff;
   __U64    e_shoff;
   __U32    e_flags;
   __U16    e_ehsize;
   __U16    e_phentsize;
   __U16    e_phnum;
   __U16    e_shentsize;
   __U16    e_shnum;
   __U16    e_shstrindx;  
}ELF64_HDR;

typedef struct{
   __U8     e_ident[EI_NIDENT];  /* Magic string */
   __U16    e_type;              /* File type, *.o, *.so or executable */
   __U16    e_machine;           /* Machine type */
   __U32    e_version;           /* version number */ 
   __U32    e_entry;             /* Entry point */   
   __U32    e_phoff;             /* Programm header offset relative file */  
   __U32    e_shoff;             /* Section header offset */
   __U32    e_flags;            
   __U16    e_ehsize;            /* ELF header size(52) */
   __U16    e_phentsize;         /* Programm header entry size */
   __U16    e_phnum;             /* Number of entries in programm header */
   __U16    e_shentsize;         /* Section header entry size */
   __U16    e_shnum;             /* Number of entries in the section header */
   __U16    e_shstrindx;         /* index of section with names of sections */
}ELF32_HDR;


/* Flags in ELF32_HDR::e_flags or ELF64_HDR::e_flags */

#define EF_MIPS_NOREORDER       0x00000001
#define EF_MIPS_PIC             0x00000002
#define EF_MIPS_CPIC            0x00000004
#define EF_MIPS_ARCH            0xF0000000


typedef struct{
   __U32    p_type;     /* programm header type */ 
   __U32    p_offset;   /* Segment file offset */
   __U32    p_vaddr;    /* Segment virtual address */
   __U32    p_paddr;    /* Segment physical address */
   __U32    p_filesz;   /* Segment size in file */
   __U32    p_memsz;    /* Segment size in memory */
   __U32    p_flags;    /* execute, read or write falgs */ 
   __U32    p_align;    /* Segment alignment, file & memory */
}ELF32_PHDR;

/* The PF_* macros are the segment flag bits int p_flags of ELFx_PHDR */
#define PF_X		0x00000001	/* Execute */
#define PF_W		0x00000002	/* Write */
#define PF_R		0x00000004	/* Read */
#define PF_MASKOS       0xFF000000      /* OS-specific */
#define PF_MASKPROC	0xF0000000	/* Processor-specific */

/* The PT_* macros are the values of p_type in ELFx_PHDR */
#define  PT_NULL	0x00000000	/* Array element is unused. Ignore */
#define  PT_LOAD	0x00000001	/* Loadable segment */
#define  PT_DYNAMIC     0x00000002      /* Dynamic segment */
#define  PT_INTERP      0x00000003      /* Section, a path to an interpreter */
#define  PT_NOTE	0x00000004      /* Note section */
#define  PT_SHLIB       0x00000005      /* Currently unspecified semantics */
#define  PT_PHDR        0x00000006      /* Programm header table */
#define  PT_LOOS        0x60000000      /* Lowest OS-specific type */  
#define  PT_HIOS        0x6FFFFFFF      /* Highest OS-specific type */
#define  PT_LOPROC      0x70000000      /* Lowest processor-specific type */
#define  PT_HIPROC      0x7FFFFFFF      /* Highest processor-specific type */


typedef struct{
   __U32    sh_name;
   __U32    sh_type;
   __U32    sh_flags;
   __U32    sh_addr;
   __U32    sh_offset;
   __U32    sh_size;
   __U32    sh_link;
   __U32    sh_info;
   __U32    sh_addralign;
   __U32    sh_entsize;     
}ELF32_SHDR;

/* sh_type */
#define  SHT_NULL	   0x00
#define  SHT_PROGBITS	   0x01
#define  SHT_SYMTAB	   0x02
#define  SHT_STRTAB    	   0x03
#define  SHT_RELA	   0x04
#define  SHT_HASH          0x05
#define  SHT_DYNAMIC       0x06
#define  SHT_NOTE          0x07
#define  SHT_NOBITS        0x08
#define  SHT_REL           0x09
#define  SHT_SHLIB         0x0A
#define  SHT_DYNSYM        0x0B
#define  SHT_NUM           0x0C
#define  SHT_LOPROC        0x70000000
#define  SHT_HIPROC        0x7FFFFFFF
#define  SHT_LOUSER 	   0x80000000
#define  SHT_HIUSER        0xffffffff
#define  SHT_MIPS_LIST     0x70000001 /* doubtful, the value was 0x70000000 */
#define  SHT_MIPS_CONFLICT 0x70000002
#define  SHT_MIPS_GPTAB    0x70000003
#define  SHT_MIPS_UCODE    0x70000004
#define  SHT_VERSYM        0x6FFFFFFF
#define  SHT_VERNEED       0x6FFFFFFE

/* sh_flags */
#define SHF_WRITE 	   0x00000001 /* Writable during execution */
#define SHF_ALLOC          0x00000002 /* Occupies memory during execution */
#define SHF_EXECINSTR      0x00000004 /* Contains executable instructions */
#define SHF_MASKPROC       0xF0000000 /* Mask for processor specific bits */ 
#define SHF_MIPS_GPREL     0x10000000


/* symbol table */

typedef struct{
   __U32 st_name;
   __U32 st_value;
   __U32 st_size; 
   __U8  st_info;
   __U8  st_other;
   __U16 st_shndx;
}ELF32_SYM;

#define ELF32_ST_BIND(i)	((i) >> 0x04) 
#define ELF32_ST_TYPE(i)	((i) & 0x0f)

/* STB_* macros are used to get the bind value from st_info */ 

#define STB_LOCAL	0x00000000
#define STB_GLOBAL	0x00000001
#define STB_WEAK	0x00000002
#define STB_LOPROC	0x0000000D
#define STB_HIPROC	0x0000000F

/* STT_* macros are used to get the type value from st_info */

#define STT_NOTYPE	0x00000000
#define STT_OBJECT	0x00000001
#define STT_FUNC	0x00000002
#define STT_SECTION	0x00000003
#define STT_FILE	0x00000004
#define STT_LOPROC	0x0000000d
#define STT_HIPROC	0x0000000F

/* SHN_* macros are used to explain the st_shndx values */

#define SHN_UNDEF	0x0000
#define SHN_LORESERVE   0xFF00
#define SHN_LOPROC	0xFF00
#define SHN_HIPROC      0xFF1F
#define SHN_ABS		0xFFF1
#define SHN_COMMON	0xFFF2
#define SHN_HIRESERVE	0xFFFF
#define SHN_MIPS_ACCOMON	0xFF00


/* Relocation section structures */

typedef struct{
   __U32	r_offset;
   __U32	r_info;	
}ELF32_REL;

typedef struct{
   __U32	r_offset;
   __U32	r_info;
   __S32	r_addend;
}ELF32_RELA;

/* Macros to find the symbol index and type or relocation */

#define ELF32_R_SYM(i) ((i) >> 0x08)
#define ELF32_R_TYPE(i) ((unsigned char)(i))

/* Macros used to define the type of relocations */

#define R_386_NONE 	0x00000000
#define R_386_32	0x00000001
#define R_386_PC32	0x00000002
#define R_386_GOT32	0x00000003
#define R_386_PLT32	0x00000004
#define R_386_COPY	0x00000005
#define R_386_GLOB_DAT	0x00000006
#define R_386_JMP_SLOT	0x00000007
#define R_386_RELATIVE	0x00000008
#define R_386_GOTOFF	0x00000009
#define R_386_GOTPC	0x0000000A
#define R_386_NUM	0x0000000B


/* Dynamic section stuff */
/* the structure which defines the entry of dynamic section */
typedef	struct{
   __S32	d_tag;        /* it defines (d_val | d_ptr) */ 
   union {
      __U32	d_val;        
      __U32     d_ptr;      
   } d_un; 
} ELF32_DYN;


/* Dynamic section stuff */
/* The macros to identify the type of dynamic entry of the dynamic section */
/* the following macros define the values of ELF32_DYN::d_tag */ 

#define DT_NULL		0
#define DT_NEEDED       1
#define DT_PLTRELSZ     2
#define DT_PLTGOT	3
#define DT_HASH		4
#define DT_STRTAB	5
#define DT_SYMTAB	6
#define DT_RELA		7
#define DT_RELASZ	8
#define DT_RELAENT	9
#define DT_STRSZ	10
#define DT_SYMENT	11
#define DT_INIT		12
#define DT_FINI		13
#define DT_SONAME	14
#define DT_RPATH	15
#define DT_SYMBOLIC	16
#define DT_REL		17
#define DT_RELSZ	18
#define DT_RELENT	19
#define DT_PLTREL 	20
#define DT_DEBUG	21
#define DT_TEXTREL	22
#define DT_JMPREL	23
#define DT_LOPROC	0x70000000
#define DT_HIPROC	0x7FFFFFFF 


/* These constants define the different elf file types for ELFX_HDR::e_type */
#define ET_NONE                 0x0000
#define ET_REL                  0x0001
#define ET_EXEC                 0x0002
#define ET_DYN                  0x0003
#define ET_CORE                 0x0004
#define ET_LOPROC               0xFF00
#define ET_HIPROC               0xFFFF


/* These macros define the various ELF target machines for the 
   ELFX_HDR::e_machine */
#define EM_NONE                 0x0000
#define EM_M32                  0x0001
#define EM_SPARC                0x0002
#define EM_386                  0x0003
#define EM_68K                  0x0004
#define EM_88K                  0x0005
#define EM_486                  0x0006  /* Perhaps disused */
#define EM_860                  0x0007
#define EM_MIPS                 0x0008  /* MIPS R3000 (Oficially big-endian) */
#define EM_MIPS_RS4_BE          0x000A  /* MIPS R4000 big-endian */
#define EM_PARISC               0x000F  /* HPPA */
#define EM_SPARC32PLUS          0x0012  /* SUN's "V8PLUS" */
#define EM_PPC                  0x0014  /* POWER PC */
#define EM_PPC64                0x0015  /* POWER PC 64 */
#define EM_SH                   0x002A  /* SUPER H */
#define EM_SPARCV9              0x002B  /* SPARC V9 64_BIT */
#define EM_IA_64                0x0032  /* HP/ INTEL IA_64 */
#define EM_X86_64               0x003E  /* AMD X86_64 */
#define EM_S390                 0x0016  /* IBM S/390 */
#define EM_CRIS                 0x004C  /* AXIS communications 32_bit */
/* 
This is an interim value that we will use untill the committee comes up with 
final number.
*/
#define EM_ALPHA                0x9026    
/*
This is the old interim value for S/39 architecture 
*/
#define EM_S390_OLD             0xA390
