/* 
dump.c 
Written by, sonimalik@users.sf.net
This module and all other modules of this package are provided under GPL
*/

#include <stdio.h>

#include "./alf.h"
#include "./stuff.h"
#include "./dumpStuff.h"

GLOBAL __U16 readSection(void){

   STACK __U32 count      = ZERO;  
   STACK __U32 localCount = ZERO; 
   STACK __U32 size       = ZERO;
   STACK __U32 memSize    = ZERO;

   STACK __U32 line       = ZERO;
   STACK __U32 lines      = ZERO;
   STACK __U32 breaks     = ZERO;


   printf("\n** *Section Dump(%s)* **\n", argumentValues [ argumentFlags[SECTION_DUMP] ]);
   printf("------------------------------------------------------\n");

/* Find the section index. */
   for(count = ZERO; count < ((ELF32_HDR *)elfHeader)->e_shnum; count++)
      if( strcmp( ((__U8 *)sectionHeaderStringBuffer +
                  ((ELF32_SHDR *)sectionHeader + count)->sh_name),                                         argumentValues[ argumentFlags[SECTION_DUMP] ] ) 
          == ZERO )

          break; 
/* ------------------------------------------------------------------------- */ 

/* See if section is found or not.*/
/* 
   If true then display the message, and replace the section name provided in
   in the command line argument with the message NOT_FOUND, this message will
   used by the code which deallocates the memory. 
*/
   if( count == ((ELF32_HDR *)elfHeader)->e_shnum ){ 
      printf("SECTION_`%s'_NOT_FOUND.\n\n", argumentValues[ 
                                          argumentFlags[SECTION_DUMP] ] ); 
      argumentValues[argumentFlags[SECTION_DUMP]] = NOT_FOUND;       
      return(NO_ERROR);		     
   }
/* ------------------------------------------------------------------------- */

/* Try to read the section into the buffer sectionDump. */  
       if( readElfEntity( argumentValues [ argumentFlags[FILE_NAME] ],
                        ((ELF32_SHDR *)sectionHeader + count)->sh_offset,
                        ((ELF32_SHDR *)sectionHeader + count)->sh_size,
                        &sectionDump ) == ERROR )
         return(ERROR);
/* ------------------------------------------------------------------------- */

/* Get the file size of the read section */ 
     size = ((ELF32_SHDR *)sectionHeader + count)->sh_size;
/* ------------------------------------------------------------------------- */

/* Get the memory size of the read section. */
     if( ((ELF32_HDR *)elfHeader)->e_shnum != (count + ONE) && 
         ((ELF32_SHDR *)sectionHeader + count + ONE)->sh_addr != ZERO ) 
        memSize = ((ELF32_SHDR *)sectionHeader + count + ONE)->sh_addr
                  - ((ELF32_SHDR *)sectionHeader + count)->sh_addr;         

/* 
   Since the size is zero there's nothing to dump, so display the appropriate
   messages.
*/
    if( size == ZERO ){
       printf("The size of the section `%s' is 0x%.8x.",
               argumentValues[argumentFlags[SECTION_DUMP]], size); 
       if( memSize != ZERO ) 
          printf("\nSection has 0x%.8x uninitialized byte/s of data.", memSize);
				                
    }
/* ------------------------------------------------------------------------- */
 
/* 
   No matter if the size is zero and there is nothing to display, 
   simply try to display the number of bytes  equal to size.
*/

/* Get the physical address of the section. */
   line = ((ELF32_SHDR *)sectionHeader + count)->sh_addr;

/* 
   Each line of the dumped data will be equal to paragraph so here we are 
   trying to get the number of such paragraphs. 
*/ 
   lines = size / SIXTEEN;
 
/* Display the dump data in hex.*/  
   for(count = ZERO; count < lines; count++){
      printf("    0x%.8x    ", line);
      for(localCount = ZERO; localCount < SIXTEEN; localCount++){    
         printf("%.2x", ((__U8 (*)[SIXTEEN])sectionDump)[count][localCount]);
         breaks += ONE;
         if( breaks == FOUR ){
            printf(" ");
            breaks = ZERO;
         }
      }
      printf("   ");
/* Display the dump data in character.*/
      for(localCount = ZERO; localCount < SIXTEEN; localCount++){
         if ( ((__U8 (*)[SIXTEEN])sectionDump)[count][localCount] >= MINIMUM_PRINT_LIMIT &&
              ((__U8 (*)[SIXTEEN])sectionDump)[count][localCount] <= MAXIMUM_PRINT_LIMIT )
         printf("%c", ((__U8 (*)[SIXTEEN])sectionDump)[count][localCount]);
         else
            printf(DOT);
      }
      line += SIXTEEN;
      printf("\n"); 
         
   }
   breaks = ZERO; 
   if( lines * SIXTEEN < size ){
      printf("    0x%.8x    ", line);
      for(count = (lines * SIXTEEN); count < size; count++){
         printf("%.2x", *((__U8 *)sectionDump + count));
         breaks+=ONE;
         if(breaks == FOUR){
           breaks = ZERO;
           printf(" ");
         }
      }

      for(count = ZERO; count < SIXTEEN - (size - (lines * SIXTEEN)); count++){
         printf("  ");  
         breaks+=ONE;
         if( breaks == FOUR ){
            breaks = ZERO;
            printf(" ");
         }
            
      }
      printf("   ");
      for(count = (lines * SIXTEEN); count < size; count++){
         if( *((__U8 *)sectionDump + count) >= 0x20 && 
             *((__U8 *)sectionDump + count) <= 0x7E    )
         printf("%c", *((__U8 *)sectionDump + count));
         else
         printf(".");
      }
     
       
   }  
 

   printf("\n\n");
   return(NO_ERROR);
}
