/* 
elfHeader.c 
Written by, sonimalik@users.sf.net
This module and all other modules of this package are provided under GPL
*/

#include <stdio.h>

#include "./alf.h"
#include "./stuff.h"
#include "./elfHeaderStuff.h"

GLOBAL void displayElfHeader(void){   

   STACK __U32 count;

printf("\n** * ELF header * **\n");
printf("--------------------\n");
/* Parse the e_ident[] array */
   /* the four byte magic string */
   printf("%x ", ((ELF32_HDR *)elfHeader)->e_ident[EI_MAG0]);
   printf("%c%c%c ", ((ELF32_HDR *)elfHeader)->e_ident[EI_MAG1], 
                    ((ELF32_HDR *)elfHeader)->e_ident[EI_MAG2],
                    ((ELF32_HDR *)elfHeader)->e_ident[EI_MAG3] );
   /* the object class, find out if its a 32 or 64 bit object */  
   switch( ((ELF32_HDR *)elfHeader)->e_ident[EI_CLASS] ){
      case ELFCLASSNONE:
         printf("`Invalid class' ");
      break;
      case ELFCLASS32:
         printf("`32 bit object' ");
      break;
      case ELFCLASS64:
         printf("`64 bit object' ");
      break;
   } 
   /* display the data encoding */
   switch( ((ELF32_HDR *)elfHeader)->e_ident[EI_DATA] ){
      case ELFDATANONE:
         printf("`Invalid data encoding' "); 
      break;
      case ELFDATA2LSB:
         printf("\"2's complement with Little Endian\" ");
      break;
      case ELFDATA2MSB:
         printf("\"2's complement with Big Endian\" "); 
      break;
   } 
   /* display the file version */
   printf("\nFile-Version                 = %d\n", 
   ((ELF32_HDR *)elfHeader)->e_ident[EI_VERSION]);   
   /* display the OS/ABI identification */
   printf("OS/ABI identification        = ");
   switch( ((ELF32_HDR *)elfHeader)->e_ident[EI_OSABI] ){
      case ELFOSABI_SYSV:
         printf("`System V ABI, third edition, no checking'");  
      break; 
      case ELFOSABI_HPUX:
         printf("`HP-UX operating system'"); 
      break;
      case ELFOSABI_STANDALONE:
         printf("`STANDALONE operating system'");
      break;            
   }  
   printf("\n");
   /* display the ABI version */
   printf("ABI-Version                  = ");
   switch( ((ELF32_HDR *)elfHeader)->e_ident[EI_OSABI] ){
      case ELFABI_HPUX_NONE:
         printf("0, `None specified'");
      break;
      case ELFABI_HPUX_REL11:
         printf("1");
      break;      
   }  
   printf("\n");
/* ------------------------------------------------------------------------ */
/* Display the file type */
   printf("File-Type                    = ");
   switch( ((ELF32_HDR *)elfHeader)->e_type ){
      case ET_NONE:
         printf("`No file type is given'\n");        
      break;
      case ET_REL:
         printf("`Relocatable/Object file'");
      break;
      case ET_EXEC:
         printf("`Executable file'");
      break;
      case ET_DYN:
         printf("`Shared object file'");
      break;
      case ET_CORE:
         printf("ET_CORE\n");
      break;
      default:
         printf("0x%x", ((ELF32_HDR *)elfHeader)->e_type);
      break;
            
   }
   printf("\n");  
/* ------------------------------------------------------------------------ */
/* Display the machine type */
/* There are many other choices which still need to be given */
   printf("Machine-Type                 = ");
   switch( ((ELF32_HDR *)elfHeader)->e_machine ){
      case EM_386:
         printf("`Intel 80386 or higher'");
      break;
   }
   printf("\n");   
/* ------------------------------------------------------------------------ */
/* Display the version */
   printf("Version                      = %d\n", 
           ((ELF32_HDR *)elfHeader)->e_version ); 
/* ------------------------------------------------------------------------ */
/* Display the entry point */
   printf("Entry-Point                  = 0x%x ", 
          ((ELF32_HDR *)elfHeader)->e_entry );
   for( count = ZERO; count < ((ELF32_HDR *)elfHeader)->e_shnum; count++ )
       if( ((ELF32_HDR *)elfHeader)->e_entry >= 
          ( (ELF32_SHDR *)sectionHeader + count)->sh_addr &&
          (((ELF32_HDR *)elfHeader)->e_entry < ((ELF32_SHDR *)sectionHeader 
           + count)->sh_addr + ((ELF32_SHDR *)sectionHeader + count)->sh_size) )
          break;  
          readString( sectionHeaderStringBuffer, 
                      ( ((ELF32_SHDR *)sectionHeader) +
                      ((ELF32_HDR *)elfHeader)->e_shstrindx)->sh_size,
                      (((ELF32_SHDR *)sectionHeader) + count)->sh_name );
   printf("\n");   
/* ------------------------------------------------------------------------ */
/* Display the programm header offset */
   printf("Programm header address      = %d(byte offset into the file)\n", 
   ((ELF32_HDR *)elfHeader)->e_phoff );
/* ------------------------------------------------------------------------ */
/* Display the section header offset */
   printf("Section header address       = %d(byte offset into the file)\n",
   ((ELF32_HDR *)elfHeader)->e_shoff );
/* ------------------------------------------------------------------------ */
/* Display the e_flags */
   printf("Flags                        = %d\n",
   ((ELF32_HDR *)elfHeader)->e_flags );
/* ------------------------------------------------------------------------ */
/* Display the elf header size */
   printf("ELF header size              = %d(bytes)\n",
   ((ELF32_HDR *)elfHeader)->e_ehsize);
/* ------------------------------------------------------------------------ */
/* Display the programm header entry size */
   printf("Programm header entry size   = %d(bytes)\n",
   ((ELF32_HDR *)elfHeader)->e_phentsize);
/* ------------------------------------------------------------------------ */
/* Display number of entries in programm header */
   printf("Entries in programm header   = %d\n",
   ((ELF32_HDR *)elfHeader)->e_phnum);
/* ------------------------------------------------------------------------ */
/* Display section header entry size */
   printf("Section Header entry size    = %d(bytes)\n",
   ((ELF32_HDR *)elfHeader)->e_shentsize);
/* ------------------------------------------------------------------------ */
/* Display number of entries in section header */
   printf("Entries in section header    = %d\n",
   ((ELF32_HDR *)elfHeader)->e_shnum);
/* ------------------------------------------------------------------------ */
/* Display the section header string table index and its name */
   printf("Section header string table  = "); 
   readString( sectionHeaderStringBuffer, 
              ( ((ELF32_SHDR *)sectionHeader) + 
              ((ELF32_HDR *)elfHeader)->e_shstrindx)->sh_size, 
              ( ((ELF32_SHDR *)sectionHeader) + 
              ((ELF32_HDR *)elfHeader)->e_shstrindx)->sh_name); 
   printf("[Index into section header = %d]\n", ((ELF32_HDR *)elfHeader)->e_shstrindx);  
/* ------------------------------------------------------------------------ */    
   printf("\n");
    		  
}

