/*
init.c
Written By, sonimalik@users.sf.net
This module and all other modules of this package are provided under GPL 
*/

#include <stdio.h>

#include "./alf.h"
#include "./stuff.h"
#include "./initStuff.h"

GLOBAL __U16 init(void){

/* Try to get the ELF32_HDR */
  if( readElfEntity( argumentValues[ argumentFlags[FILE_NAME] ], 
  ZERO, sizeof(ELF32_HDR), &elfHeader) == ERROR )
     return(ERROR);
/* ------------------------------------------------------------------------- */

/* Try to get the Section Header */
  if( readElfEntity( argumentValues[ argumentFlags[FILE_NAME] ], 
                    ((ELF32_HDR *)elfHeader)->e_shoff, 
                    sizeof(ELF32_SHDR) * ((ELF32_HDR *)elfHeader)->e_shnum,
                    &sectionHeader) == ERROR )
     return(ERROR);  
/* ------------------------------------------------------------------------- */
 
/* Try to get the Programm Header */
     if( readElfEntity( argumentValues[ argumentFlags[FILE_NAME] ],
                        ((ELF32_HDR *)elfHeader)->e_phoff,
                        sizeof(ELF32_PHDR) * ((ELF32_HDR *)elfHeader)->e_phnum,
                        &programmHeader ) == ERROR )
        return(ERROR);   
/* ------------------------------------------------------------------------- */                
/* Try to get the section header string table */
  if( readElfEntity( argumentValues[ argumentFlags[FILE_NAME] ],
                   ( ((ELF32_SHDR *)sectionHeader) + 
                   ((ELF32_HDR *)elfHeader)->e_shstrindx)->sh_offset, 
                   ( ((ELF32_SHDR *)sectionHeader) + 
                   ((ELF32_HDR *)elfHeader)->e_shstrindx)->sh_size, 
                   &sectionHeaderStringBuffer) == ERROR )            
     return(ERROR);
/* ------------------------------------------------------------------------ */

    if( display() == ERROR ){
       printf("\n\nSOME THING BAD HAS HAPPENED :-(\n\n"); 
       exit(ERROR);
    }


/* Try to release the memory blocks */
  allocateOrFreeMemory( &elfHeader, ZERO );
  allocateOrFreeMemory( &programmHeader, ZERO );
  allocateOrFreeMemory( &sectionHeader, ZERO ); 
  allocateOrFreeMemory( &sectionHeaderStringBuffer, ZERO );
  if( argumentFlags[SECTION_DUMP] > ZERO 
     && strcmp( argumentValues[argumentFlags[SECTION_DUMP]], 
        NOT_FOUND)  != ZERO ) 
     allocateOrFreeMemory( &sectionDump, ZERO );

  if( argumentFlags[DYNAMIC] > ZERO 
     && strcmp( argumentValues[argumentFlags[DYNAMIC]],
        NOT_FOUND) != ZERO )
     allocateOrFreeMemory( &dynamicSection, ZERO );

        
  return(NO_ERROR);   
}

GLOBAL __U16 readString(REGISTER_OR_STACK __U8 * buffer,
                        REGISTER_OR_STACK __U32 bufferSize,
                        REGISTER_OR_STACK __U32 index){

   __U32 count = ZERO;
   
   for(count = ZERO; count < bufferSize; count++)
      if( count == index ){ 
         printf("%s ", (buffer + count));
         return(NO_ERROR); 
      }
      

   return(ERROR);

} 


GLOBAL __U16 readElfEntity(REGISTER_OR_STACK void * fileName, 
                           REGISTER_OR_STACK __U32 entityOffset, 
                           REGISTER_OR_STACK __U32 entitySize, 
                           REGISTER_OR_STACK void ** buffer){
    
   void * justToMoveFilePointer = ZERO;    
 
     
   if( openOrCloseFile(&fileDescriptor, fileName) == ERROR )
      return(ERROR);                    

   if( entityOffset > ZERO ){
      if( allocateOrFreeMemory(&justToMoveFilePointer, entityOffset) == ERROR )
         return(ERROR);
      if( readFile(&fileDescriptor, 
      &justToMoveFilePointer, entityOffset) == ERROR )
         return(ERROR);
      if( allocateOrFreeMemory(&justToMoveFilePointer, ZERO ) == ERROR)
         return(ERROR);
   }
   
   if( allocateOrFreeMemory(buffer, entitySize) == ERROR )
      return(ERROR);
  
   if( readFile(&fileDescriptor, buffer, entitySize) == ERROR )
      return(ERROR);  
             
   if( openOrCloseFile(&fileDescriptor, "") == ERROR )
      return(ERROR);

   
   return(NO_ERROR);
}


GLOBAL __U16 openOrCloseFile(REGISTER_OR_STACK void * fileDescriptor, 
                             REGISTER_OR_STACK void * fileName){
 
   if( *((__S32 *)fileDescriptor) == ZERO )
      *((__S32 *)fileDescriptor) = open(fileName, O_RDWR);    
   else
      if( close ( *((__S32 *)fileDescriptor) ) < ZERO ){
         printf("Unable to close %s\n", fileName); 
         return(ERROR);      
      }
      else
         *((__S32 *)fileDescriptor) = ZERO;
              
   if( *((__S32 *)fileDescriptor) < ZERO ){
      printf("Unable to open %s\n", fileName);
      return(ERROR);
   }
  
   return(NO_ERROR); 
 
}

GLOBAL __U16 readFile(REGISTER_OR_STACK void * fileDescriptor,
                      REGISTER_OR_STACK void ** buffer,
                      REGISTER_OR_STACK __U32 bufferLength){

   if(read( *((__U32 *)fileDescriptor), *buffer, bufferLength) == NEGATIVE_ONE){
      printf("Unable to read file.\n");
      return(ERROR);   
   }

   return(NO_ERROR);      

}

GLOBAL __U16 allocateOrFreeMemory(REGISTER_OR_STACK void ** buffer, 
                                  REGISTER_OR_STACK __U32 bufferLength){

   if( bufferLength != ZERO ){
      *buffer = malloc(bufferLength);
      if( *buffer == NULL ){
         printf("Unable to allocate memory.\n");         
         return(ERROR);
      }
   }
   else
      free(*buffer);
     
   return(NO_ERROR);

}
