/*
relocation.c
Written by, sonimalik@users.sf.net
This module and all other modules of this package are provided under GPL
*/

#include <stdio.h>

#include "./alf.h"
#include "./stuff.h"
#include "./relocationStuff.h"

GLOBAL __U16 relocation(void){

  STACK void * relocations;
  STACK void * symbols;
  STACK void * strings;

  STACK __U32 count      = ZERO; 
  STACK __U32 localCount = ZERO;

  for(count = ZERO; count < ((ELF32_HDR *)elfHeader)->e_shnum; count++)
     if( ((ELF32_SHDR *)sectionHeader + count)->sh_type == SHT_REL ||
         ((ELF32_SHDR *)sectionHeader + count)->sh_type == SHT_REL ) 
         localCount++;  
 
 
  if( localCount > ZERO ){
     printf("** *Relocation Sections* **\n");
     printf("---------------------------\n");
     printf("There are %d such sections.\n\n", localCount);
  }
  else{
     printf("NO_RELOCATION_SECTIONS\n");  
     return(ZERO); 
  }
  for( count = ZERO; count < ((ELF32_HDR *)elfHeader)->e_shnum; count++ ){

     if( ((ELF32_SHDR *)sectionHeader + count)->sh_type == SHT_REL ||
         ((ELF32_SHDR *)sectionHeader + count)->sh_type == SHT_RELA   ){

        if(  readElfEntity( argumentValues[ argumentFlags[FILE_NAME] ],
                            ((ELF32_SHDR *)sectionHeader + count)->sh_offset,
                            ((ELF32_SHDR *)sectionHeader + count)->sh_size,
                            &relocations) == ERROR){
           printf("Unable to read, relocation section.\n");
           return(ERROR); 
        }

        if( readElfEntity( argumentValues[ argumentFlags[FILE_NAME] ],
                   ((ELF32_SHDR *)sectionHeader + 
                   ((ELF32_SHDR *)sectionHeader + count)->sh_link)->sh_offset,
                   ((ELF32_SHDR *)sectionHeader +
                   ((ELF32_SHDR *)sectionHeader + count)->sh_link)->sh_size,
                   &symbols) == ERROR){
           printf("Unable to read, symbol table.\n");
           return(ERROR); 
        } 

        if( readElfEntity (argumentValues[ argumentFlags[FILE_NAME] ],
                       ((ELF32_SHDR *)sectionHeader + 
                       ((ELF32_SHDR *)sectionHeader +  
                       ((ELF32_SHDR *)sectionHeader + 
                       count)->sh_link)->sh_link)->sh_offset,
                       ((ELF32_SHDR *)sectionHeader + 
                       ((ELF32_SHDR *)sectionHeader + 
                       ((ELF32_SHDR *)sectionHeader + 
                       count)->sh_link)->sh_link)->sh_size,
                       &strings) == ERROR ){
           printf("Unable to read, string table.\n"); 
           return(ERROR);
        }
        
        printf("relocation section `"); 
        readString( sectionHeaderStringBuffer, 
                    ((ELF32_SHDR *)sectionHeader + 
                    ((ELF32_HDR *)elfHeader)->e_shstrindx )->sh_size, 
                    ((ELF32_SHDR *)sectionHeader + count)->sh_name     ); 
        printf("' is of type `");
        switch( ((ELF32_SHDR *)sectionHeader + count)->sh_type ){
           case SHT_REL:
              printf("SHT_REL");
           break;
           case SHT_RELA:           
              printf("SHT_RELA");
           break;
        } 
        printf("'; it has %d entries. ", 
                ((ELF32_SHDR *)sectionHeader + count)->sh_size / 
                ((ELF32_SHDR *)sectionHeader + count)->sh_entsize);
 
        if( ((ELF32_SHDR *)sectionHeader + count)->sh_info != ZERO ){
           printf("It affects the section `");
           readString( sectionHeaderStringBuffer,
                       ((ELF32_SHDR *)sectionHeader + 
                       ((ELF32_HDR *)elfHeader)->e_shstrindx)->sh_size,  
                       ((ELF32_SHDR *)sectionHeader + 
                       ((ELF32_SHDR *)sectionHeader+count)->sh_info)->sh_name);
           printf("'. "); 
          
        }
       printf("The symbol table, linked to this relocation section is `");
       readString(sectionHeaderStringBuffer,
                  ((ELF32_SHDR *)sectionHeader +
                  ((ELF32_HDR *)elfHeader)->e_shstrindx)->sh_size, 
                  ((ELF32_SHDR *)sectionHeader + 
                  ((ELF32_SHDR *)sectionHeader + count)->sh_link)->sh_name );
       printf("'. The string table, linked to the symbol table is `");
       
       readString(sectionHeaderStringBuffer,
                  ((ELF32_SHDR *)sectionHeader +
                  ((ELF32_HDR *)elfHeader)->e_shstrindx)->sh_size,
                  
                  ((ELF32_SHDR *)sectionHeader +                                                  ( ((ELF32_SHDR *)sectionHeader +
                    ((ELF32_SHDR *)sectionHeader + count)->sh_link)->sh_link )
                  )->sh_name );
       printf("'.\n\n");  
 
       printf("OFFSET   ADDEND   TYPE           SYM_ADDR SYM_SIZE SYM_BIND SYMBOL(SYM)\n");
       for(localCount = ZERO; localCount <
           ((ELF32_SHDR *)sectionHeader + count)->sh_size;){

           printf("%.8x",((ELF32_REL *)(
                         (__U8 *)relocations + localCount))->r_offset );

           if( ((ELF32_SHDR *)sectionHeader + count)->sh_type == SHT_RELA)
              printf(" %.8x", ((ELF32_RELA *)(
                              (__U8 *)relocations + localCount))->r_addend );
           else
              printf(" (none)  ");  

           switch( ELF32_R_TYPE( ((ELF32_REL *)(
                   (__U8 *)relocations + localCount))->r_info) ){
              case R_386_NONE:
                 printf(" R_386_NONE    ");
              break;
              case R_386_32:
                 printf(" R_386_32      ");
              break; 
              case R_386_PC32:
                 printf(" R_386_PC32    "); 
              break;
              case R_386_GOT32:
                 printf(" R_386_GOT32   ");
              break;
              case R_386_PLT32:
                 printf(" R_386_PLT32   "); 
              break;
              case R_386_COPY:
                 printf(" R_386_COPY    ");
              break;
              case R_386_GLOB_DAT:
                 printf(" R_386_GLOB_DAT");
              break; 
              case R_386_JMP_SLOT:
                 printf(" R_386_JMP_SLOT");
              break;
              case R_386_RELATIVE:
                 printf(" R_386_RELATIVE"); 
              break;
              case R_386_GOTOFF:
                 printf(" R_386_GOTOFF  ");
              break;
              case R_386_GOTPC:
                 printf(" R_386_GOTPC   ");
              break;
              case R_386_NUM:
                 printf(" R_386_NUM     ");
              break;  
           }
           
           printf(" %.8x", ((ELF32_SYM *)symbols + ELF32_R_SYM( ((ELF32_REL *)((__U8 *)relocations + localCount))->r_info))->st_value ); 
           printf(" %.8x", ((ELF32_SYM *)symbols + ELF32_R_SYM( ((ELF32_REL *)((__U8 *)relocations + localCount))->r_info))->st_size );
           switch( ELF32_ST_BIND( ((ELF32_SYM *)symbols + ELF32_R_SYM( ((ELF32_REL *)((__U8 *)relocations + localCount))->r_info))->st_info) ){
              case STB_LOCAL:
                 printf(" LOCAL   "); 
              break;
              case STB_GLOBAL:
                 printf(" GLOBAL  "); 
              break;
              case STB_WEAK:
                 printf(" WEAK    ");
              break;
           }
           printf(" ");
           /*
            if we've a valid ELF32_SYM::st_shndx.
            It is the section where the name is defined, like data which
            has externel or internel linkage and is uninitialized goes in 
            the .bss and likewise if it initialized goes to the .data section.
           */             
           if( ((ELF32_SYM *)symbols + ELF32_R_SYM( ((ELF32_REL *)((__U8 *)relocations + localCount))->r_info))->st_shndx < SHN_LORESERVE && ((ELF32_SYM *)symbols + ELF32_R_SYM( ((ELF32_REL *)((__U8 *)relocations + localCount))->r_info))->st_shndx != SHN_UNDEF ){
           printf("["); 
           readString( sectionHeaderStringBuffer, 
                       ((ELF32_SHDR *)sectionHeader + 
                       ((ELF32_HDR *)elfHeader)->e_shstrindx )->sh_size, 
                       ((ELF32_SHDR *)sectionHeader + 
               ((ELF32_SYM *)symbols + 
                       ELF32_R_SYM( 
                       ((ELF32_REL *)((__U8 *)relocations 
                       + localCount))->r_info) 
               )->st_shndx)->sh_name ); 
           printf("]");
           }
           /*
            ELF32_SYM::st_shndx is invalid, 
           */
           else{
              getSectionName( ((ELF32_SYM *)symbols + ELF32_R_SYM( ((ELF32_REL *)((__U8 *)relocations + localCount))->r_info))->st_value ); 

           }

           readString( strings,            
                       ((ELF32_SHDR *)sectionHeader + 
                       ((ELF32_SHDR *)sectionHeader + 
                       ((ELF32_SHDR *)sectionHeader + 
                       count)->sh_link)->sh_link)->sh_size, 
                       ((ELF32_SYM *)symbols + 
                       ELF32_R_SYM( ((ELF32_REL *)(
                       (__U8 *)relocations + localCount))->r_info) )->st_name);

            printf("--->");  
            getSectionName( ((ELF32_REL *)((__U8 *)relocations + localCount))->r_offset );
           
           localCount+=((ELF32_SHDR *)sectionHeader + count)->sh_entsize;

           printf("\n"); 
 
       }
                                           
        printf("\n");
        printf("Notes:\n");
        printf("Note 1: [.sectionName ] = ELFX_SYM::st_shndx. (index of a section where the symbol is defined)\n");
        printf("Note 2: (.sectionName ) = <ELFX_REL::r_offset | \ 
ELFX_SYM::st_value> is between {ELFX_SHDR[x]::sh_addr & (ELFX_SHDR[x]::sh_addr + \ 
ELFX_SHDR[x]::sh_size)}(inclusive). ELFX_SYM::st_shndx is not used, since it is \
either equal to or greater than SHN_LORESERVE.\n");
         printf("Note 3: {.sectionName } = <ELFX_REL::r_offset | \ 
ELFX_SYM::st_value> is in between {ELFX_SHDR[0...(n-1)th]::sh_size  \ 
and (ELFX_SHDR[0...(n-1)th] + ELFX_SHDR[nth]::sh_size)}. \ 
ELFX_SHDR::sh_addr is not used since it is zero and ELFX_SYM::st_shndx \ 
is equal to or beyound SHN_LORESERVE.\n");
        printf("\n");
        allocateOrFreeMemory(&relocations, ZERO);
        allocateOrFreeMemory(&symbols, ZERO);
        allocateOrFreeMemory(&strings, ZERO);
 
     }  
      
  }

   return(NO_ERROR);
}


LOCAL __U16 getSectionName(__U32 address){

  STACK  __U16 count      = ZERO;
  STACK  __U16 localCount = ZERO;
  STACK  __U32 size       = ZERO;

  if(address == ZERO)
     return(ZERO);

  for(count = ZERO; count < ((ELF32_HDR *)elfHeader)->e_shnum; count++){
     if( ((ELF32_SHDR *)sectionHeader + count)->sh_addr != ZERO){ 

     if( address >= ((ELF32_SHDR *)sectionHeader + count)->sh_addr &&
         address <= ( ((ELF32_SHDR *)sectionHeader + count)->sh_addr + 
                      ((ELF32_SHDR *)sectionHeader + count)->sh_size - 1 ) ){
        printf("(");
        readString(sectionHeaderStringBuffer, 
                   ((ELF32_SHDR *)sectionHeader + 
                   ((ELF32_HDR *)elfHeader)->e_shstrindx)->sh_size,
                   ((ELF32_SHDR *)sectionHeader + count)->sh_name);
       printf(")");
        
     }
     }

     else{
         for(localCount = ZERO; localCount < count; localCount++){
         size+=((ELF32_SHDR *)sectionHeader + localCount)->sh_size; 
         }
         if( address > size && address <= size + ((ELF32_SHDR *)sectionHeader + count)->sh_size ){
            printf("{");
            readString(sectionHeaderStringBuffer,
                       ((ELF32_SHDR *)sectionHeader + 
                       ((ELF32_HDR *)elfHeader)->e_shstrindx)->sh_size,
                       ((ELF32_SHDR *)sectionHeader + count)->sh_name); 
            printf("}");
        }
     }
    
             
   
  }  
    
  return(NO_ERROR);
} 


