/* 
sections.c
Written by, sonimalik@users.sf.net
This module and all other modules of this package are provided under GPL
*/

#include <stdio.h>

#include "./alf.h"
#include "./stuff.h"
#include "./sectionsStuff.h"


GLOBAL __U16 sections(void){

   STACK __U32 count      = ZERO; 
   STACK __U32 localCount = ZERO;
   STACK __U32 nameSize   = ZERO;

   printf("\n** *Section Headers* **\n"); 
   printf("-----------------------\n");
   for(count = ZERO; count < ((ELF32_HDR *)elfHeader)->e_shnum; count++){

     if( strlen(  (  (__U8 *)sectionHeaderStringBuffer + 
                ((ELF32_SHDR *)sectionHeader + count)->sh_name ) ) 
                > nameSize )
        nameSize = strlen ( ( (__U8 *)sectionHeaderStringBuffer + 
                          ((ELF32_SHDR *)sectionHeader + count)->sh_name ) ); 
      
   }
   printf("NAME"); 
   for( count = ZERO; count < (nameSize - FOUR); count++)
       printf(" ");  
   printf(" TYPE     ADDR     OFFSET   MEM_SIZE ALIGN    ENT_SIZE FLAGS");
   printf("\n"); 
   for( count = ZERO; count < ((ELF32_HDR *)elfHeader)->e_shnum; count++){
       
       readString( sectionHeaderStringBuffer, 
                  ((ELF32_SHDR *)sectionHeader + 
                  ((ELF32_HDR *)elfHeader)->e_shstrindx)->sh_size, 
                  ((ELF32_SHDR *)sectionHeader + count)->sh_name);   

       for(localCount = ZERO;
           localCount <(nameSize - strlen ( ( (__U8 *)sectionHeaderStringBuffer 
                                   + ((ELF32_SHDR *)sectionHeader 
                                                    + count)->sh_name ) ) );
           localCount++)
           printf(" ");

             

       switch( ((ELF32_SHDR *)sectionHeader + count)->sh_type ){
          case SHT_NULL:
             printf("NULL     ");
          break;
          case SHT_PROGBITS:
             printf("PROGBITS ");
          break;
          case SHT_SYMTAB:
             printf("SYMTAB   ");
          break;
          case SHT_STRTAB:
             printf("STRTAB   ");
          break;
          case SHT_RELA:
             printf("RELA     ");
          break;
          case SHT_HASH:
             printf("HASH     ");
          break;
          case SHT_DYNAMIC:
             printf("DYNAMIC  "); 
          break;
          case SHT_NOTE:
             printf("NOTE     ");
          break;
          case SHT_NOBITS:
             printf("NOBITS   ");
          break;
          case SHT_REL:
             printf("REL      ");
          break;
          case SHT_SHLIB:
             printf("SHLIB    "); 
          break;
          case SHT_DYNSYM:
             printf("DYNSYM   ");
          break;
          case SHT_NUM:
             printf("NUM      ");
          break; 
          case SHT_VERSYM:
             printf("VERSYM   ");
          break;
          case SHT_VERNEED:
             printf("VERNEED  ");
          break;
          default:
             printf("!        ");
          break;
       }
           
       printf("%.8x ", ((ELF32_SHDR *)sectionHeader + count)->sh_addr); 
       printf("%.8x ", ((ELF32_SHDR *)sectionHeader + count)->sh_offset);
       printf("%.8x ", ((ELF32_SHDR *)sectionHeader + count)->sh_size);
       printf("%.8x ", ((ELF32_SHDR *)sectionHeader + count)->sh_addralign);
       printf("%.8x ", ((ELF32_SHDR *)sectionHeader + count)->sh_entsize);
       
       if( ((ELF32_SHDR *)sectionHeader + count)->sh_flags & SHF_WRITE )
          printf("W");
       if( ((ELF32_SHDR *)sectionHeader + count)->sh_flags & SHF_ALLOC )
          printf("A");
       if( ((ELF32_SHDR *)sectionHeader + count)->sh_flags & SHF_EXECINSTR )
          printf("X");
       if( ((ELF32_SHDR *)sectionHeader + count)->sh_flags & SHF_MASKPROC )
          printf("M");
       if( ((ELF32_SHDR *)sectionHeader + count)->sh_flags & SHF_MIPS_GPREL )
          printf("I");
    
       printf("\n");        
   }

   printf("\n");
   printf("ELFX_SHDR::sh_link:\n");
   for( count = ZERO; count < ((ELF32_HDR *)elfHeader)->e_shnum; count++ )
       if( ((ELF32_SHDR *)sectionHeader + count)->sh_link > ZERO ){
      
          printf("Section ");

           readString( sectionHeaderStringBuffer, 
                  ((ELF32_SHDR *)sectionHeader + 
                  ((ELF32_HDR *)elfHeader)->e_shstrindx)->sh_size, 
                  ((ELF32_SHDR *)sectionHeader + 
                  ((ELF32_SHDR *)sectionHeader + count)->sh_link)->sh_name);   

           printf("is linked to the section ");
    
           readString( sectionHeaderStringBuffer, 
                  ((ELF32_SHDR *)sectionHeader + 
                  ((ELF32_HDR *)elfHeader)->e_shstrindx)->sh_size, 
                  ((ELF32_SHDR *)sectionHeader + count)->sh_name);   

           printf("\n");
       }
        
    
   printf("\n");
   printf("ELFX_SHDR::sh_info:\n");
   for( count = ZERO; count < ((ELF32_HDR *)elfHeader)->e_shnum; count++ )
       if( ((ELF32_SHDR *)sectionHeader + count)->sh_info > ZERO && 
          ( ((ELF32_SHDR *)sectionHeader + count)->sh_type == SHT_RELA ||
            ((ELF32_SHDR *)sectionHeader + count)->sh_type == SHT_REL   ) ){
      
           printf("Section ");

           readString( sectionHeaderStringBuffer, 
                  ((ELF32_SHDR *)sectionHeader + 
                  ((ELF32_HDR *)elfHeader)->e_shstrindx)->sh_size, 
                  ((ELF32_SHDR *)sectionHeader + 
                  ((ELF32_SHDR *)sectionHeader + count)->sh_info)->sh_name);   

           printf("is affected by the section ");
    
           readString( sectionHeaderStringBuffer, 
                  ((ELF32_SHDR *)sectionHeader + 
                  ((ELF32_HDR *)elfHeader)->e_shstrindx)->sh_size, 
                  ((ELF32_SHDR *)sectionHeader + count)->sh_name);   

           printf("\n");
       }             
                
   printf("\nDescription of ELFX_SHDR::sh_flags:\n");
   printf("W = writable during execution\n");
   printf("A = occupies memory during execution\n");
   printf("X = contains executable instructions\n"); 
   printf("\n\n");
   return(NO_ERROR);
}


