/*
 symbols.c 
 Written by, sonimalik@users.sf.net
 This module and the rest of the modules are provided under GPL
*/

#include <stdio.h>

#include "alf.h"
#include "stuff.h"
#include "symbolsStuff.h"

GLOBAL __U16 symbols(void){

   STACK void* symbols;
   STACK void* strings;

   STACK __U32 count      = ZERO;
   STACK __U32 localCount = ZERO;

   for(;count<((ELF32_HDR *)elfHeader)->e_shnum;){
      if(((ELF32_SHDR *)sectionHeader + count)->sh_type == SHT_SYMTAB 
         || ((ELF32_SHDR *)sectionHeader + count)->sh_type == SHT_DYNSYM )   
         localCount++;      
      count++;
   }
   if(localCount > ZERO){      
      printf("** *Symbol Table Sections* **\n");
      printf("-----------------------------\n");
      printf("There is/are %d such sections.\n\n",localCount);
   }
   else{
      printf("NO_SYMBOL_SECTIONS\n");
      return(ZERO);   
   }

   for(count=ZERO; count<((ELF32_HDR*)elfHeader)->e_shnum; count++){
      if(((ELF32_SHDR *)sectionHeader + count)->sh_type == SHT_SYMTAB 
         || ((ELF32_SHDR *)sectionHeader + count)->sh_type == SHT_DYNSYM ){

         if(  readElfEntity( argumentValues[ argumentFlags[FILE_NAME] ],
                           ((ELF32_SHDR *)sectionHeader + count)->sh_offset,
                           ((ELF32_SHDR *)sectionHeader + count)->sh_size,
                           &symbols) == ERROR){
          printf("Unable to read, symbol section.\n");
          return(ERROR);
        }

        if( readElfEntity( argumentValues[ argumentFlags[FILE_NAME] ],
           ((ELF32_SHDR*)sectionHeader +  ((ELF32_SHDR *)sectionHeader + count)->sh_link)->sh_offset,
           ((ELF32_SHDR*)sectionHeader +  ((ELF32_SHDR *)sectionHeader + count)->sh_link)->sh_size,
           &strings) == ERROR){
           printf("Unable to read, string section.\n");
           allocateOrFreeMemory(&symbols, ZERO);
           return(ERROR);
        }

        printf("Symbol table `");
        readString(sectionHeaderStringBuffer, 
                  ((ELF32_SHDR *)sectionHeader + 
                  ((ELF32_HDR *)elfHeader)->e_shstrindx )->sh_size, 
                  ((ELF32_SHDR *)sectionHeader + count)->sh_name     ); 
        printf("' is of type `");
        switch(((ELF32_SHDR*)sectionHeader + count)->sh_type){
           case SHT_SYMTAB:
              printf("SHT_SYMTAB"); 
           break;
           case SHT_DYNSYM:
              printf("SHT_DYNSYM");  
           break;
           default:
              printf("unknown");
           break;
        }
        printf("' and it has %d entries.\n",
              ((ELF32_SHDR*)sectionHeader + count)->sh_size/((ELF32_SHDR*)sectionHeader + count)->sh_entsize);
        printf("The string table linked to this section is `");
        readString(sectionHeaderStringBuffer, 
                  ((ELF32_SHDR *)sectionHeader + 
                  ((ELF32_HDR *)elfHeader)->e_shstrindx )->sh_size, 
                  ((ELF32_SHDR*)sectionHeader + ((ELF32_SHDR *)sectionHeader + count)->sh_link)->sh_name); 
        printf("'\n\n");
        printf("VALUE    SIZE(10) TYPE     BIND    SYMBOL\n");
        localCount = ZERO;   
        for(;localCount<((ELF32_SHDR*)sectionHeader + count)->sh_size/((ELF32_SHDR*)sectionHeader + count)->sh_entsize;){

           printf("%.8x",((ELF32_SYM*)symbols + localCount)->st_value);
           printf(" %.8u",((ELF32_SYM*)symbols + localCount)->st_size);

           switch(ELF32_ST_TYPE(((ELF32_SYM*)symbols + localCount)->st_info)){
              case STT_NOTYPE:
                 printf(" NOTYPE  "); 
              break;
              case STT_OBJECT:
                 printf(" OBJECT  ");
              break;           
              case STT_FUNC:
                 printf(" FUNC    "); 
              break;
              case STT_SECTION:
                 printf(" SECTION ");
              break; 
              case STT_FILE:
                 printf(" FILE    ");
              break;
              default:
                 printf(" --------"); 
              break; 
           }           
           switch(ELF32_ST_BIND(((ELF32_SYM*)symbols + localCount)->st_info)){
              case STB_LOCAL:
                 printf(" LOCAL   ");
              break;
              case STB_GLOBAL:
                 printf(" GLOBAL  ");
              break;
              case STB_WEAK:
                 printf(" WEAK    ");
              break; 
              default:
                 printf(" --------");  
              break;
           }
/*           printf(" ");*/
           switch(((ELF32_SYM*)symbols + localCount)->st_shndx){
              case SHN_UNDEF:
                 printf("(UNDEF)"); 
              break;
              case SHN_ABS:
                 printf("(ABS)");
              break;
              case SHN_COMMON:
                 printf("(COMMON)");
              break;
              case SHN_MIPS_ACCOMON:
              break;
              default:
/*                 if( ((ELF32_SYM*)symbols + localCount)->st_shndx >= SHN_LORESERVE && ((ELF32_SYM*)symbols + localCount)->st_shndx <= SHN_HIRESERVE ){*/
                    printf("[");
                    readString(sectionHeaderStringBuffer, 
                              ((ELF32_SHDR *)sectionHeader + 
                              ((ELF32_HDR *)elfHeader)->e_shstrindx )->sh_size, 
                              ((ELF32_SHDR*)sectionHeader + ((ELF32_SYM*)symbols + localCount)->st_shndx)->sh_name); 
                    printf("]"); 
/*                 }*/
              break;   
           }
/*
           if(((ELF32_SYM*)symbols + localCount)->st_shndx > 0){
             printf("[");
             readString(sectionHeaderStringBuffer, 
                  ((ELF32_SHDR *)sectionHeader + 
                  ((ELF32_HDR *)elfHeader)->e_shstrindx )->sh_size, 
                  ((ELF32_SHDR*)sectionHeader + ((ELF32_SYM*)symbols + localCount)->st_shndx)->sh_name); 
             printf("]"); 
           }
*/

           readString(strings, 
           ((ELF32_SHDR*)sectionHeader +  ((ELF32_SHDR *)sectionHeader + count)->sh_link)->sh_size,
           ((ELF32_SYM*)symbols + localCount)->st_name );


           printf("\n");  
           localCount+=1;     
        }
                       
        allocateOrFreeMemory(&symbols, ZERO);
        allocateOrFreeMemory(&strings, ZERO);
        printf("\nNotes:\n");
        printf("Note 1: [.sectionName ] = ELFX_SYM::st_shndx. (Index of the section where symbol is defined)\n");          
        
   printf("\n");    

      }

   } 

   return(NO_ERROR);
}
